Include "Lib_Matrix.bb"
Include "Lib_Control.bb"

;Global
Type Entity
  Field Matrix.Matrix, vis, typ

  Field X#, Y#, Z#
  Field sx#, sy#, sz#
  Field Pitch#, Yaw#
  Field col1, col2, col3
  Field Fliped

  Field Zoom#, ProjMode
  Field ClsColor1, ClsColor2, ClsColor3
  Field ViewX, ViewY, ViewWidth, ViewHeight
End Type
Type Vertex
  Field Entity.Entity
  Field x#, y#, z#
  Field px, py
  Field Vector3D.Vector
  Field col1, col2, col3
End Type
Type Polygon
  Field Entity.Entity
  Field v1.Vertex, v2.Vertex, v3.Vertex
  Field Distance#
End Type
Global Mat1.Matrix = CreateMatrix()
Global Mat2.Matrix = CreateMatrix()
Global Mat3.Matrix = CreateMatrix()
Global Mat4.Matrix = CreateMatrix()
Const TypeCamera = 1
Global ScreenWidth, ScreenHeight
Dim xval(3), yval(3)
Dim Tmp.Vertex(1000000), Tmp2.Vertex(1024, 1024), Tmp3.Vertex(100, 100, 4)
Global Tris
Global Wire = False
Global winLeft
Global winRight
Global winTop
Global winBottom

Function rInit(width, height, depth, windowmode)
Graphics width, height, depth, windowmode
SetBuffer BackBuffer()
SeedRnd MilliSecs()
ScreenWidth = width
ScreenHeight = height
End Function

Function rRenderWorld()
;Position&Rotate Objects
For e.Entity = Each Entity
  MatrixRotateY(Mat2, -e\Yaw#)
  MatrixRotateX(Mat3, -e\Pitch#)
  MatrixScale(Mat4, e\sx#, e\sz#, e\sz#)
  MatrixMultiplyMatrix(Mat2, Mat3, e\Matrix)
  MatrixMultiplyMatrix(Mat4, e\Matrix, e\Matrix)
Next
;Render
Tris = 0
For c.Entity = Each Entity
  If c\typ = TypeCamera And c\vis And c\ProjMode <> 0 Then
    ;Set ViewPort Options
    winLeft = c\ViewX
    winTop = c\ViewY
    winRight = c\ViewWidth + c\ViewX - 1
    winBottom = c\ViewHeight + c\ViewY - 1
    Color c\ClsColor1 * c\col1 / 255, c\ClsColor2 * c\col2 / 255, c\ClsColor3 * c\col3 / 255
    Rect c\ViewX, c\ViewY, c\ViewWidth, c\ViewHeight
    LockBuffer BackBuffer()
    ;Translate Camera
    MatrixTranslate(Mat1, -c\X#, -c\Y#, -c\Z#)
    MatrixMultiplyMatrix(Mat1, c\Matrix, c\Matrix)
    ;Project Vertices
    For v.Vertex = Each Vertex
      ;Transpose Vertex to Vector3D
      v\Vector3D\x# = v\x#
      v\Vector3D\y# = v\y#
      v\Vector3D\z# = v\z#
      ;Vector * Entity * Camera
      VectorMultiplyMatrix(v\Vector3D, v\Entity\Matrix)
      VectorMultiplyMatrix(v\Vector3D, c\Matrix)
      ;Project Vertices
      Select c\ProjMode
        Case 1
          v\px = Render_ProjectX(v\Vector3D\x# + v\Entity\X#, v\Vector3D\z# + v\Entity\Z#, c\Zoom#, c\ViewWidth, c\ViewX)
          v\py = Render_ProjectY(v\Vector3D\y# + v\Entity\Y#, v\Vector3D\z# + v\Entity\Z#, c\Zoom#, c\ViewWidth, c\ViewY - c\ViewWidth / 8)
        Case 2
          v\px = Render_ProjectX(v\Vector3D\x# + v\Entity\X#, 1, c\Zoom#, c\ViewWidth, c\ViewX)
          v\py = Render_ProjectY(v\Vector3D\y# + v\Entity\Y#, 1, c\Zoom#, c\ViewWidth, c\ViewY - c\ViewWidth / 8)
      End Select
    Next
    ;Sort Polygons
    If Wire = False Then
      cnt = 0
      For p.Polygon = Each Polygon
        VX# = (p\v1\Vector3D\x# + p\v2\Vector3D\x# + p\v3\Vector3D\x#) / 3 + p\Entity\X#
        VY# = (p\v1\Vector3D\y# + p\v2\Vector3D\y# + p\v3\Vector3D\y#) / 3 + p\Entity\Y#
        VZ# = (p\v1\Vector3D\z# + p\v2\Vector3D\z# + p\v3\Vector3D\z#) / 3 + p\Entity\Z#
        p\Distance# = VX# * VX# + VY# * VY# + VZ# * VZ#
        cnt = cnt + 1
      Next
      If cnt > 1 Then
        bs = 1
        Repeat
          p.Polygon = Last Polygon
          Insert p Before First Polygon
          p.Polygon = First Polygon
          value# = p\Distance#
          x = 1
          bs = bs + 1
          While bs <> x
            p.Polygon = After p
            x = x + 1
            If value# < p\Distance# Then
              Insert p Before Before p
              p.Polygon = After p
            Else
              Exit
            End If
          Wend
        Until cnt = bs
      EndIf
    EndIf
    ;Draw Triangles
    For p.Polygon = Each Polygon
      If p\Entity\vis Then
        If Render_Cull(p\v1\px, p\v1\py, p\v2\px, p\v2\py, p\v3\px, p\v3\py) Xor p\Entity\Fliped And (p\v1\px > 0 Or p\v2\px > 0 Or p\v3\px > 0) And (p\v1\py > 0 Or p\v2\py > 0 Or p\v3\py > 0) And (p\v1\px < ScreenWidth Or p\v2\px < ScreenWidth Or p\v3\px < ScreenWidth) And (p\v1\py < ScreenHeight Or p\v2\py < ScreenHeight Or p\v3\py < ScreenHeight) And p\v1\Vector3D\z# + p\Entity\Z# > 0 And p\v2\Vector3D\z# + p\Entity\Z# > 0 And p\v3\Vector3D\z# + p\Entity\Z# > 0 Then
          If Wire Then
            Color 255, 255, 255
            Line p\v1\px, p\v1\py, p\v2\px, p\v2\py
            Line p\v2\px, p\v2\py, p\v3\px, p\v3\py
            Line p\v3\px, p\v3\py, p\v1\px, p\v1\py
          Else
            sc1# = c\col1 * p\Entity\col1 / 65025.0
            sc2# = c\col2 * p\Entity\col2 / 65025.0
            sc3# = c\col3 * p\Entity\col3 / 65025.0
            Render_DrawTriangle(p\v1\px, p\v1\py, p\v2\px, p\v2\py, p\v3\px, p\v3\py, p\v1\col1 * sc1, p\v1\col2 * sc2, p\v1\col3 * sc3, p\v2\col1 * sc1, p\v2\col2 * sc2, p\v2\col3 * sc3, p\v3\col1 * sc1, p\v3\col2 * sc2, p\v3\col3 * sc3)
          EndIf
          Tris = Tris + 1
        EndIf
      EndIf
    Next
    UnlockBuffer BackBuffer()
  EndIf
Next
End Function

Function Render_ProjectX(x#, z#, zoom#, ViewportWidth, ViewPortX)
Return x# * ViewportWidth * zoom# / z# / 2 + ViewportWidth / 2 + ViewPortX
End Function

Function Render_ProjectY(y#, z#, zoom#, ViewportHeight, ViewPortY)
Return y# * ViewportHeight * -zoom# / z# / 2 + ViewportHeight / 2 + ViewPortY
End Function

Function Render_Cull(x1, y1, x2, y2, x3, y3)
If x1 * y3 - x1 * y2 - x2 * y3 + x2 * y2 - y1 * x3 + y1 * x2 + y2 * x3 - y2 * x2 < 0 Then Return True Else Return False
End Function

Function Render_DrawTriangle(x0, y0, x1, y1, x2, y2, r0, g0, b0, r1, g1, b1, r2, g2, b2)
If y0 > y1 Then
  tmpx = y0
  y0 = y1
  y1 = tmpx
  tmpx = x0
  x0 = x1
  x1 = tmpx
  tmpx = r0
  r0 = r1
  r1 = tmpx
  tmpx = g0
  g0 = g1
  g1 = tmpx
  tmpx = b0
  b0 = b1
  b1 = tmpx
End If
If y0 > y2 Then
  tmpx = y0
  y0 = y2
  y2 = tmpx
  tmpx = x0
  x0 = x2
  x2 = tmpx
  tmpx = r0
  r0 = r2
  r2 = tmpx
  tmpx = g0
  g0 = g2
  g2 = tmpx
  tmpx = b0
  b0 = b2
  b2 = tmpx
End If
If y1 > y2 Then
  tmpx = y1
  y1 = y2
  y2 = tmpx
  tmpx = x1
  x1 = x2
  x2 = tmpx
  tmpx = r1
  r1 = r2
  r2 = tmpx
  tmpx = g1
  g1 = g2
  g2 = tmpx
  tmpx = b1
  b1 = b2
  b2 = tmpx
End If
If y0 <> y1 Then
  If y1 = y2 Then ey = y1 Else ey = y1 - 1
  dx1 = x1 - x0
  dy1 = y1 - y0
  df1 = dx1 / dy1
  dm1 = Abs(dx1 Mod dy1)
  dx2 = x2 - x0
  dy2 = y2 - y0
  df2 = dx2 / dy2
  dm2 = Abs(dx2 Mod dy2)
  drg1 = r1 - r0
  dfrg1 = drg1 / dy1
  dmrg1 = Abs(drg1 Mod dy1)
  dgg1 = g1 - g0
  dfgg1 = dgg1 / dy1
  dmgg1 = Abs(dgg1 Mod dy1)
  dbg1 = b1 - b0
  dfbg1 = dbg1 / dy1
  dmbg1 = Abs(dbg1 Mod dy1)
  drg2 = r2 - r0
  dfrg2 = drg2 / dy2
  dmrg2 = Abs(drg2 Mod dy2)
  dgg2 = g2 - g0
  dfgg2 = dgg2 / dy2
  dmgg2 = Abs(dgg2 Mod dy2)
  dbg2 = b2 - b0
  dfbg2 = dbg2 / dy2
  dmbg2 = Abs(dbg2 Mod dy2)
  If x0 > x2 Then xs2 = -1 Else xs2 = 1
  If x0 > x1 Then xs1 = -1 Else xs1 = 1
  If r0 > r1 Then rsg1 = -1 Else rsg1 = 1
  If g0 > g1 Then gsg1 = -1 Else gsg1 = 1
  If b0 > b1 Then bsg1 = -1 Else bsg1 = 1
  If r0 > r2 Then rsg2 = -1 Else rsg2 = 1
  If g0 > g2 Then gsg2 = -1 Else gsg2 = 1
  If b0 > b2 Then bsg2 = -1 Else bsg2 = 1
  xp1 = x0
  mc1 = dm1
  xp2 = x0
  mc2 = dm2
  rg1 = r0
  mcrg1 = dmrg1
  gg1 = g0
  mcgg1 = dmgg1
  bg1 = b0
  mcbg1 = dmbg1
  rg2 = r0
  mcrg2 = dmrg2
  gg2 = g0
  mcgg2 = dmgg2
  bg2 = b0
  mcbg2 = dmbg2
  For yp = y0 To ey
    xc1 = xp1
    xc2 = xp2
    rcg1 = rg1
    gcg1 = gg1
    bcg1 = bg1
    rcg2 = rg2
    gcg2 = gg2
    bcg2 = bg2
    If xc1 <> xc2 And yp => winTop And yp <= winBottom Then
      If xc1 > xc2 Then
        tmpx = xc1
        xc1 = xc2
        xc2 = tmpx
        tmpx = rcg1
        rcg1 = rcg2
        rcg2 = tmpx
        tmpx = gcg1
        gcg1 = gcg2
        gcg2 = tmpx
        tmpx = bcg1
        bcg1 = bcg2
        bcg2 = tmpx
      End If
      If xc1 <= winRight And xc2 => winLeft Then
        d = xc2 - xc1
        If xc1 < winLeft Then xc1 = winLeft
        If xc2 > winRight Then xc2 = winRight
        If rcg1 > rcg2 Then rs = -1 Else rs = 1
        If gcg1 > gcg2 Then gs = -1 Else gs = 1
        If bcg1 > bcg2 Then bs = -1 Else bs = 1
        dr = rcg2 - rcg1
        dfr = dr / d
        dmr = Abs(dr Mod d)
        dg = gcg2 - gcg1
        dfg = dg / d
        dmg = Abs(dg Mod d)
        db = bcg2 - bcg1
        dfb = db / d
        dmb = Abs(db Mod d)
        r = rcg1
        mcr = dmr
        g = gcg1
        mcg = dmg
        b = bcg1
        mcb = dmb
        For xp = xc1 To xc2
          WritePixelFast xp, yp, (r Shl 16) Or (g Shl 8) Or b, BackBuffer()
          r = r + dfr
          g = g + dfg
          b = b + dfb
          mcr = mcr + dmr
          If mcr => d Then
            mcr = mcr - d
            r = r + rs
          End If
          mcg = mcg + dmg
          If mcg => d Then
            mcg = mcg - d
            g = g + gs
          End If
          mcb = mcb + dmb
          If mcb => d Then
            mcb = mcb - d
            b = b + bs
          End If
        Next
      End If
    End If
    xp1 = xp1 + df1
    xp2 = xp2 + df2
    rg1 = rg1 + dfrg1
    gg1 = gg1 + dfgg1
    bg1 = bg1 + dfbg1
    rg2 = rg2 + dfrg2
    gg2 = gg2 + dfgg2
    bg2 = bg2 + dfbg2
    mc1 = mc1 + dm1
    mc2 = mc2 + dm2
    mcrg1 = mcrg1 + dmrg1
    mcgg1 = mcgg1 + dmgg1
    mcbg1 = mcbg1 + dmbg1
    mcrg2 = mcrg2 + dmrg2
    mcgg2 = mcgg2 + dmgg2
    mcbg2 = mcbg2 + dmbg2
    If mc1 => dy1 Then
      mc1 = mc1 - dy1
      xp1 = xp1 + xs1
    End If
    If mc2 => dy2 Then
      mc2 = mc2 - dy2
      xp2 = xp2 + xs2
    End If
    If mcrg1 => dy1 Then
      mcrg1 = mcrg1 - dy1
      rg1 = rg1 + rsg1
    End If
    If mcgg1 => dy1 Then
      mcgg1 = mcgg1 - dy1
      gg1 = gg1 + gsg1
    End If
    If mcbg1 => dy1 Then
      mcbg1 = mcbg1 - dy1
      bg1 = bg1 + bsg1
    End If
    If mcrg2 => dy2 Then
      mcrg2 = mcrg2 - dy2
      rg2 = rg2 + rsg2
    End If
    If mcgg2 => dy2 Then
      mcgg2 = mcgg2 - dy2
      gg2 = gg2 + gsg2
    End If
    If mcbg2 => dy2 Then
      mcbg2 = mcbg2 - dy2
      bg2 = bg2 + bsg2
    End If
  Next
Else
  If y1 = y2 Then Return
  dx2 = x2 - x0
  dy2 = y2 - y0
  df2 = dx2 / dy2
  dm2 = Abs(dx2 Mod dy2)
  drg2 = r2 - r0
  dfrg2 = drg2 / dy2
  dmrg2 = Abs(drg2 Mod dy2)
  dgg2 = g2 - g0
  dfgg2 = dgg2 / dy2
  dmgg2 = Abs(dgg2 Mod dy2)
  dbg2 = b2 - b0
  dfbg2 = dbg2 / dy2
  dmbg2 = Abs(dbg2 Mod dy2)
  If x0 > x2 Then xs2 = -1 Else xs2 = 1
  If r0 > r2 Then rsg2 = -1 Else rsg2 = 1
  If g0 > g2 Then gsg2 = -1 Else gsg2 = 1
  If b0 > b2 Then bsg2 = -1 Else bsg2 = 1
  xp2 = x0
  mc2 = dm2
  rg2 = r0
  mcrg2 = dmrg2
  gg2 = g0
  mcgg2 = dmgg2
  bg2 = b0
  mcbg2 = dmbg2
End If
If y1 <> y2 Then
  dx1 = x2 - x1
  dy1 = y2 - y1
  df1 = dx1 / dy1
  dm1 = Abs(dx1 Mod dy1)
  drg1 = r2 - r1
  dfrg1 = drg1 / dy1
  dmrg1 = Abs(drg1 Mod dy1)
  dgg1 = g2 - g1
  dfgg1 = dgg1 / dy1
  dmgg1 = Abs(dgg1 Mod dy1)
  dbg1 = b2 - b1
  dfbg1 = dbg1 / dy1
  dmbg1 = Abs(dbg1 Mod dy1)
  If x1 > x2 Then xs1 = -1 Else xs1 = 1
  If r1 > r2 Then rsg1 = -1 Else rsg1 = 1
  If g1 > g2 Then gsg1 = -1 Else gsg1 = 1
  If b1 > b2 Then bsg1 = -1 Else bsg1 = 1
  xp1 = x1
  mc1 = dm1
  rg1 = r1
  mcrg1 = dmrg1
  gg1 = g1
  mcgg1 = dmgg1
  bg1 = b1
  mcbg1 = dmbg1
  If y2 > winBottom Then y2 = winBottom
  For yp = y1 To y2
    xc1 = xp1
    xc2 = xp2
    rcg1 = rg1
    gcg1 = gg1
    bcg1 = bg1
    rcg2 = rg2
    gcg2 = gg2
    bcg2 = bg2
    If xc1 <> xc2 And yp => winTop And yp <= winBottom Then
      If xc1 > xc2 Then
        tmpx = xc1
        xc1 = xc2
        xc2 = tmpx
        tmpx = rcg1
        rcg1 = rcg2
        rcg2 = tmpx
        tmpx = gcg1
        gcg1 = gcg2
        gcg2 = tmpx
        tmpx = bcg1
        bcg1 = bcg2
        bcg2 = tmpx
      End If
      If xc1 <= winRight And xc2 => winLeft Then
        d = xc2 - xc1
        If xc1 < winLeft Then xc1 = winLeft
        If xc2 > winRight Then xc2 = winRight
        If rcg1 > rcg2 Then rs = -1 Else rs = 1
        If gcg1 > gcg2 Then gs = -1 Else gs = 1
        If bcg1 > bcg2 Then bs = -1 Else bs = 1
        dr = rcg2 - rcg1
        dfr = dr / d
        dmr = Abs(dr Mod d)
        dg = gcg2 - gcg1
        dfg = dg / d
        dmg = Abs(dg Mod d)
        db = bcg2 - bcg1
        dfb = db / d
        dmb = Abs(db Mod d)
        r = rcg1
        mcr = dmr
        g = gcg1
        mcg = dmg
        b = bcg1
        mcb = dmb
        For xp = xc1 To xc2
          WritePixelFast xp, yp, (r Shl 16) Or (g Shl 8) Or b, BackBuffer()
          r = r + dfr
          g = g + dfg
          b = b + dfb
          mcr = mcr + dmr
          If mcr => d Then
            mcr = mcr - d
            r = r + rs
          End If
          mcg = mcg + dmg
          If mcg => d Then
            mcg = mcg - d
            g = g + gs
          End If
          mcb = mcb + dmb
          If mcb => d Then
            mcb = mcb - d
            b = b + bs
          End If
        Next
      End If
    End If
    xp1 = xp1 + df1
    xp2 = xp2 + df2
    rg1 = rg1 + dfrg1
    gg1 = gg1 + dfgg1
    bg1 = bg1 + dfbg1
    rg2 = rg2 + dfrg2
    gg2 = gg2 + dfgg2
    bg2 = bg2 + dfbg2
    mc1 = mc1 + dm1
    mc2 = mc2 + dm2
    mcrg1 = mcrg1 + dmrg1
    mcgg1 = mcgg1 + dmgg1
    mcbg1 = mcbg1 + dmbg1
    mcrg2 = mcrg2 + dmrg2
    mcgg2 = mcgg2 + dmgg2
    mcbg2 = mcbg2 + dmbg2
    If mc1 => dy1 Then
      mc1 = mc1 - dy1
      xp1 = xp1 + xs1
    End If
    If mc2 => dy2 Then
      mc2 = mc2 - dy2
      xp2 = xp2 + xs2
    End If
    If mcrg1 => dy1 Then
      mcrg1 = mcrg1 - dy1
      rg1 = rg1 + rsg1
    End If
    If mcgg1 => dy1 Then
      mcgg1 = mcgg1 - dy1
      gg1 = gg1 + gsg1
    End If
    If mcbg1 => dy1 Then
      mcbg1 = mcbg1 - dy1
      bg1 = bg1 + bsg1
    End If
    If mcrg2 => dy2 Then
      mcrg2 = mcrg2 - dy2
      rg2 = rg2 + rsg2
    End If
    If mcgg2 => dy2 Then
      mcgg2 = mcgg2 - dy2
      gg2 = gg2 + gsg2
    End If
    If mcbg2 => dy2 Then
      mcbg2 = mcbg2 - dy2
      bg2 = bg2 + bsg2
    End If
  Next
End If
End Function

Function rAddTriangle.Polygon(Mesh.Entity, v1.Vertex, v2.Vertex, v3.Vertex)
p.Polygon = New Polygon
p\Entity = Mesh
p\v1 = v1
p\v2 = v2
p\v3 = v3
Return p
End Function

Function rAddVertex.Vertex(Mesh.Entity, x#, y#, z#)
v.Vertex = New Vertex
v\Entity.Entity = Mesh
v\x# = x#
v\y# = y#
v\z# = z#
v\col1 = 255
v\col2 = 255
v\col3 = 255
v\Vector3D = CreateVector()
Return v
End Function

Function rCreateMesh.Entity()
e.Entity = New Entity
e\Matrix.Matrix = CreateMatrix()
e\sx# = 1
e\sy# = 1
e\sz# = 1
e\vis = True
e\Zoom# = 1
e\ProjMode = 1
e\col1 = 255
e\col2 = 255
e\col3 = 255
Return e
End Function

Function rCreateCube.Entity()
Mesh.Entity = rCreateMesh()
v1.Vertex = rAddVertex(Mesh, -.5, -.5, -.5)
v2.Vertex = rAddVertex(Mesh, .5, -.5, -.5)
v3.Vertex = rAddVertex(Mesh, -.5, -.5, .5)
v4.Vertex = rAddVertex(Mesh, .5, -.5, .5)
v5.Vertex = rAddVertex(Mesh, -.5, .5, -.5)
v6.Vertex = rAddVertex(Mesh, .5, .5, -.5)
v7.Vertex = rAddVertex(Mesh, -.5, .5, .5)
v8.Vertex = rAddVertex(Mesh, .5, .5, .5)
rAddTriangle(Mesh, v1, v5, v6)
rAddTriangle(Mesh, v1, v6, v2)
rAddTriangle(Mesh, v1, v7, v5)
rAddTriangle(Mesh, v1, v3, v7)
rAddTriangle(Mesh, v6, v8, v2)
rAddTriangle(Mesh, v4, v2, v8)
rAddTriangle(Mesh, v7, v3, v8)
rAddTriangle(Mesh, v3, v4, v8)
rAddTriangle(Mesh, v5, v7, v8)
rAddTriangle(Mesh, v5, v8, v6)
rAddTriangle(Mesh, v1, v2, v3)
rAddTriangle(Mesh, v2, v4, v3)
Return Mesh
End Function

Function rCreateCone.Entity(seg# = 8)
If seg < 3 Or seg > 100 Then RuntimeError("Illegal number of segments.")
Mesh.Entity = rCreateMesh()
For i# = 1 To seg#
  w# = 360 * i# / seg#
  Tmp(i) = rAddVertex(Mesh, Cos(w#) / 2, -.5, Sin(w#) / 2)
Next
v1.Vertex = rAddVertex(Mesh, 0, .5, 0)
For i = 1 To seg#
  a = i + 1
  If a > seg Then a = 1
  rAddTriangle(Mesh, Tmp(a), Tmp(i), v1)
Next
For i = 1 To seg - 2
  a = i + 1: If a > seg Then a = 1
  b = i + 2: If b > seg Then b = 2
  rAddTriangle(Mesh, Tmp(1), Tmp(a), Tmp(b))
Next
Return Mesh
End Function

Function rCreateCylinder.Entity(seg# = 8)
If seg < 3 Or seg > 100 Then RuntimeError("Illegal number of segments.")
Mesh.Entity = rCreateMesh()
For i# = 1 To seg#
  w# = 360 * i# / seg#
  Tmp2(i, 0) = rAddVertex(Mesh, Cos(w#) / 2, -.5, Sin(w#) / 2)
  Tmp2(i, 1) = rAddVertex(Mesh, Cos(w#) / 2, .5, Sin(w#) / 2)
Next
For i = 1 To seg#
  a = i + 1
  If a > seg Then a = 1
  rAddTriangle(Mesh, Tmp2(a, 0), Tmp2(i, 0), Tmp2(a, 1))
  rAddTriangle(Mesh, Tmp2(i, 0), Tmp2(i, 1), Tmp2(a, 1))
Next
For i = 1 To seg - 2
  a = i + 1: If a > seg Then a = 1
  b = i + 2: If b > seg Then b = 2
  rAddTriangle(Mesh, Tmp2(1, 0), Tmp2(a, 0), Tmp2(b, 0))
  rAddTriangle(Mesh, Tmp2(1, 1), Tmp2(b, 1), Tmp2(a, 1))
Next
Return Mesh
End Function

Function rCreateFrustrum.Entity(seg# = 8, rad# = .5)
If seg < 3 Or seg > 100 Then RuntimeError("Illegal number of segments.")
Mesh.Entity = rCreateMesh()
For i# = 1 To seg#
  w# = 360 * i# / seg#
  Tmp2(i, 0) = rAddVertex(Mesh, Cos(w#) / 2, -.5, Sin(w#) / 2)
  Tmp2(i, 1) = rAddVertex(Mesh, Cos(w#) * rad# / 2, .5, Sin(w#) * rad# / 2)
Next
For i = 1 To seg#
  a = i + 1
  If a > seg Then a = 1
  rAddTriangle(Mesh, Tmp2(a, 0), Tmp2(i, 0), Tmp2(a, 1))
  rAddTriangle(Mesh, Tmp2(i, 0), Tmp2(i, 1), Tmp2(a, 1))
Next
For i = 1 To seg - 2
  a = i + 1: If a > seg Then a = 1
  b = i + 2: If b > seg Then b = 2
  rAddTriangle(Mesh, Tmp2(1, 0), Tmp2(a, 0), Tmp2(b, 0))
  rAddTriangle(Mesh, Tmp2(1, 1), Tmp2(b, 1), Tmp2(a, 1))
Next
Return Mesh
End Function

Function rCreateTube.Entity(seg# = 8, rad# = .5)
If seg < 3 Or seg > 100 Then RuntimeError("Illegal number of segments.")
Mesh.Entity = rCreateMesh()
For i# = 1 To seg#
  w# = 360 * i# / seg#
  Tmp2(i, 0) = rAddVertex(Mesh, Cos(w#) / 2, -.5, Sin(w#) / 2)
  Tmp2(i, 1) = rAddVertex(Mesh, Cos(w#) / 2, .5, Sin(w#) / 2)
  Tmp2(i, 2) = rAddVertex(Mesh, Cos(w#) * rad# / 2, -.5, Sin(w#) * rad# / 2)
  Tmp2(i, 3) = rAddVertex(Mesh, Cos(w#) * rad# / 2, .5, Sin(w#) * rad# / 2)
Next
For i = 1 To seg#
  a = i + 1
  If a > seg Then a = 1
  rAddTriangle(Mesh, Tmp2(a, 0), Tmp2(i, 0), Tmp2(a, 1))
  rAddTriangle(Mesh, Tmp2(i, 0), Tmp2(i, 1), Tmp2(a, 1))
  rAddTriangle(Mesh, Tmp2(a, 2), Tmp2(a, 3), Tmp2(i, 2))
  rAddTriangle(Mesh, Tmp2(i, 2), Tmp2(a, 3), Tmp2(i, 3))
  rAddTriangle(Mesh, Tmp2(i, 0), Tmp2(a, 0), Tmp2(i, 2))
  rAddTriangle(Mesh, Tmp2(i, 2), Tmp2(a, 0), Tmp2(a, 2))
  rAddTriangle(Mesh, Tmp2(i, 1), Tmp2(i, 3), Tmp2(a, 1))
  rAddTriangle(Mesh, Tmp2(i, 3), Tmp2(a, 3), Tmp2(a, 1))
Next
Return Mesh
End Function

Function rCreateTorus.Entity(seg# = 8, rad# = .5)
If seg < 3 Or seg > 100 Then RuntimeError("Illegal number of segments.")
Mesh.Entity = rCreateMesh()
For cnt1 = 1 To seg#
  fat# = cnt1 * 360.0 / seg#
  radius# = 1 + rad# * Sin(fat)
  deg# = 0
  z# = rad# * Cos(fat)
  For cnt2 = 1 To seg#
    deg# = cnt2 * 360.0 / seg#
	  x# = radius# * Cos(deg)
    y# = radius# * Sin(deg)
    Tmp2(cnt1, cnt2) = rAddVertex(Mesh, x# / 2, y# / 2, z# / 2)
  Next
Next
For x = 1 To seg#
  For y = 1 To seg#
    a = x + 1
    b = y + 1
    If a > seg# Then a = 1
    If b > seg# Then b = 1
    rAddTriangle Mesh, Tmp2(x, y), Tmp2(a, y), Tmp2(x, b)
    rAddTriangle Mesh, Tmp2(a, b), Tmp2(x, b), Tmp2(a, y)
  Next
Next
Return Mesh
End Function

Function rCreateSphere.Entity(seg# = 8)
If seg < 3 Or seg > 100 Then RuntimeError("Illegal number of segments.")
Mesh.Entity = rCreateMesh()
div# = 180.0 / seg
height# = 1.0
RotAngle# = 0
For i = 1 To seg * 2
  np.Vertex = rAddVertex(Mesh, 0, height / 2, 0)
  sp.Vertex = rAddVertex(Mesh, 0, -height / 2, 0)
  YPos# = Cos(div#)
  XPos# = -Cos(RotAngle#) * Sin(div#)
  ZPos# = Sin(RotAngle#) * Sin(div#)
  v0t.Vertex = rAddVertex(Mesh, XPos# / 2, YPos# / 2, ZPos# / 2)
  v0b.Vertex = rAddVertex(Mesh, XPos# / 2, -YPos# / 2, ZPos# / 2)
  RotAngle# = RotAngle# + div#
  XPos# = -Cos(RotAngle#) * Sin(div#)
  ZPos# = Sin(RotAngle#) * Sin(div#)
  upos# = upos# - udiv#
  v1t.Vertex = rAddVertex(Mesh, XPos# / 2, YPos# / 2, ZPos# / 2)
  v1b.Vertex = rAddVertex(Mesh, XPos# / 2, -YPos# / 2, ZPos# / 2)
  rAddTriangle Mesh, np, v0t, v1t
  rAddTriangle Mesh, v1b, v0b, sp
Next
RotAngle# = 0
For i = 1 To seg * 2
  mult# = 1
  YPos# = Cos(div# * mult#)
  YPos2# = Cos(div# * mult# + div#)
  Thisvdiv# = vdiv#
  For j = 1 To seg - 2
    XPos# = -Cos(RotAngle#) * Sin(div# * mult#)
    ZPos# = Sin(RotAngle#) * Sin(div# * mult#)
    XPos2# = -Cos(RotAngle#) * Sin(div# * mult# + div#)
    ZPos2# = Sin(RotAngle#) * Sin(div# * mult# + div#)
    v0t.Vertex = rAddVertex(Mesh, XPos# / 2, YPos# / 2, ZPos# / 2)
    v0b.Vertex = rAddVertex(Mesh, XPos2# / 2, YPos2# / 2, ZPos2# / 2)
    tempRotAngle# = RotAngle# + div#
    XPos# = -Cos(tempRotAngle#) * Sin(div# * mult#)
    ZPos# = Sin(tempRotAngle#) * Sin(div# * mult#)
    XPos2# = -Cos(tempRotAngle#) * Sin(div# * mult# + div#)
    ZPos2# = Sin(tempRotAngle#) * Sin(div# * mult# + div#)
    v1t.Vertex = rAddVertex(Mesh, XPos# / 2, YPos# / 2, ZPos# / 2)
    v1b.Vertex = rAddVertex(Mesh, XPos2# / 2, YPos2# / 2, ZPos2# / 2)
    rAddTriangle Mesh, v1t, v0t, v0b
    rAddTriangle Mesh, v1b, v1t, v0b
    mult# = mult# + 1
    YPos# = Cos(div# * mult#)
    YPos2# = Cos(div# * mult# + div#)
  Next
  RotAngle# = RotAngle# + div#
Next
Return Mesh
End Function

Function rCreateCamera.Entity()
e.Entity = rCreateMesh()
e\typ = TypeCamera
e\ViewWidth = ScreenWidth
e\ViewHeight = ScreenHeight
Return e
End Function

Function rLoadMesh.Entity(path$)
Mesh.Entity = rCreateMesh()
file = ReadFile(path$)
VCnt = ReadInt(file)
TCnt = ReadInt(file)
For v = 0 To VCnt
  Tmp(v) = rAddVertex(Mesh, ReadFloat(file), ReadFloat(file), ReadFloat(file))
Next
For t = 0 To TCnt
  rAddTriangle(Mesh, Tmp(ReadInt(file)), Tmp(ReadInt(file)), Tmp(ReadInt(file)))
Next
Return Mesh
End Function

Function UpdateDebugLine(x = 0, y = 0)
Color 0, 0, 0
Rect x, y, 98, 25
Color 255, 255, 255
Rect x - 1, y - 1, 100, 27, False
Text x + 5, y + 5,  "FPS = " + Int(FPS)
End Function