Type Matrix
  Field aa#, ab#, ac#, ad#
  Field ba#, bb#, bc#, bd#
  Field ca#, cb#, cc#, cd#
  Field da#, db#, dc#, dd#
End Type
Type Vector
  Field x#, y#, z#
End Type

Function CreateMatrix.Matrix()
Local m.Matrix = New Matrix
MatrixIdentity(m)
Return m
End Function

Function FreeMatrix(m.Matrix)
Delete m
m = Null
End Function

Function MatrixIdentity(m.Matrix)
m\aa# = 1: m\ab# = 0: m\ac# = 0: m\ad# = 0
m\ba# = 0: m\bb# = 1: m\bc# = 0: m\bd# = 0
m\ca# = 0: m\cb# = 0: m\cc# = 1: m\cd# = 0
m\da# = 0: m\db# = 0: m\dc# = 0: m\dd# = 1
End Function

Function MatrixRotateX(m.Matrix, a#)
m\aa# = 1: m\ab# = 0:         m\ac# = 0:        m\ad# = 0
m\ba# = 0: m\bb# = Cos#(a#):  m\bc# = Sin#(a#): m\bd# = 0
m\ca# = 0: m\cb# = -Sin#(a#): m\cc# = Cos#(a#): m\cd# = 0
m\da# = 0: m\db# = 0:         m\dc# = 0:        m\dd# = 1
End Function

Function MatrixRotateY(m.Matrix, a#)
m\aa# = Cos#(a#): m\ab# = 0: m\ac# = -Sin#(a#): m\ad# = 0
m\ba# = 0:        m\bb# = 1: m\bc# = 0:         m\bd# = 0
m\ca# = Sin#(a#): m\cb# = 0: m\cc# = Cos#(a#):  m\cd# = 0
m\da# = 0:        m\db# = 0: m\dc# = 0:         m\dd# = 1
End Function

Function MatrixRotateZ(m.Matrix, a#)
m\aa# =  Cos#(a#): m\ab# = Sin#(a#): m\ac# = 0: m\ad# = 0
m\ba# = -Sin#(a#): m\bb# = Cos#(a#): m\bc# = 0: m\bd# = 0
m\ca# =  0:        m\cb# = 0:        m\cc# = 1: m\cd# = 0
m\da# =  0:        m\db# = 0:        m\dc# = 0: m\dd# = 1
End Function

Function MatrixScale(m.Matrix, sx#, sy#, sz#)
m\aa# = sx#: m\ab# = 0:   m\ac# = 0:   m\ad# = 0
m\ba# = 0:   m\bb# = sy#: m\bc# = 0:   m\bd# = 0
m\ca# = 0:   m\cb# = 0:   m\cc# = sz#: m\cd# = 0
m\da# = 0:   m\db# = 0:   m\dc# = 0:   m\dd# = 1
End Function

Function MatrixTranslate(m.Matrix, tx#, ty#, tz#)
m\aa# = 1:   m\ab# = 0:   m\ac# = 0:   m\ad# = 0
m\ba# = 0:   m\bb# = 1:   m\bc# = 0:   m\bd# = 0
m\ca# = 0:   m\cb# = 0:   m\cc# = 1:   m\cd# = 0
m\da# = tx#: m\db# = ty#: m\dc# = tz#: m\dd# = 1
End Function

Function MatrixMultiplyMatrix(a.Matrix, b.Matrix, c.Matrix)
c\aa# = a\aa# * b\aa# + a\ab# * b\ba# + a\ac# * b\ca# + a\ad# * b\da#
c\ab# = a\aa# * b\ab# + a\ab# * b\bb# + a\ac# * b\cb# + a\ad# * b\db#
c\ac# = a\aa# * b\ac# + a\ab# * b\bc# + a\ac# * b\cc# + a\ad# * b\dc#
c\ad# = a\aa# * b\ad# + a\ab# * b\bd# + a\ac# * b\cd# + a\ad# * b\dd#
c\ba# = a\ba# * b\aa# + a\bb# * b\ba# + a\bc# * b\ca# + a\bd# * b\da#
c\bb# = a\ba# * b\ab# + a\bb# * b\bb# + a\bc# * b\cb# + a\bd# * b\db#
c\bc# = a\ba# * b\ac# + a\bb# * b\bc# + a\bc# * b\cc# + a\bd# * b\dc#
c\bd# = a\ba# * b\ad# + a\bb# * b\bd# + a\bc# * b\cd# + a\bd# * b\dd#
c\ca# = a\ca# * b\aa# + a\cb# * b\ba# + a\cc# * b\ca# + a\cd# * b\da#
c\cb# = a\ca# * b\ab# + a\cb# * b\bb# + a\cc# * b\cb# + a\cd# * b\db#
c\cc# = a\ca# * b\ac# + a\cb# * b\bc# + a\cc# * b\cc# + a\cd# * b\dc#
c\cd# = a\ca# * b\ad# + a\cb# * b\bd# + a\cc# * b\cd# + a\cd# * b\dd#
c\da# = a\da# * b\aa# + a\db# * b\ba# + a\dc# * b\ca# + a\dd# * b\da#
c\db# = a\da# * b\ab# + a\db# * b\bb# + a\dc# * b\cb# + a\dd# * b\db#
c\dc# = a\da# * b\ac# + a\db# * b\bc# + a\dc# * b\cc# + a\dd# * b\dc#
c\dd# = a\da# * b\ad# + a\db# * b\bd# + a\dc# * b\cd# + a\dd# * b\dd#
End Function

Function CreateVector.Vector()
Local v.Vector = New Vector
Return v
End Function

Function VectorMultiplyMatrix(v.Vector, m.Matrix)
tx# = v\x#
ty# = v\y#
tz# = v\z#
v\x# = m\aa# * tx# + m\ba# * ty# + m\ca# * tz# + m\da#
v\y# = m\ab# * tx# + m\bb# * ty# + m\cb# * tz# + m\db#
v\z# = m\ac# * tx# + m\bc# * ty# + m\cc# * tz# + m\dc#
End Function